from enum import Enum


class TestProjectFiles(Enum):
    CONFIGS_DIR = "configs"
    CONFIG_FILE = "config.py"
    PAGE_OBJECT_DIR = "page_object"
    PAGES_DIR = "pages"
    POPUPS_DIR = "popups"
    BASE_PAGE = "base_page.py"
    BASE_POPUP = "base_popup.py"
    SERVICES_DIR = "services"
    BASE_SERVICE = "base_service.py"
    TESTS_DIR = "tests"
    BASE_TEST = "base_test.py"
    TOOLS_FOLDER = "tools"
    UTILS_FOLDER = "utils"



from common_components.src.test_project_files import \
    TestProjectFiles as TP
from common_components.src.base_files_dir import\
    BasePythonFileDir as BF
from common_components.src.descriptions import\
    Descriptions


class TestProjectSampleTree:

    @property
    def dir_name(self):
        return self.root_dir_name

    def __init__(self, root_dir_name=None):
        self.root_dir_name = root_dir_name

    @property
    def tree(self):
        return self.tree_dict()

    def tree_dict(self):
        return {
            "name": self.root_dir_name,
            "type": "directory",
            "children": [
                {
                    "name": TP.TOOLS_FOLDER.value,
                    "type": "directory",
                },
                {
                    "name": TP.TESTS_DIR.value,
                    "type": "directory",
                },
                {
                    "name": TP.PAGE_OBJECT_DIR.value,
                    "type": "directory",
                    "children": [
                        {
                            "name": TP.POPUPS_DIR.value,
                            "type": "directory",
                        },
                        {
                            "name": TP.PAGES_DIR.value,
                            "type": "directory",
                        }
                    ]
                },
                {
                    "name": TP.CONFIGS_DIR.value,
                    "type": "directory",
                },
                {
                    "name": TP.SERVICES_DIR.value,
                    "type": "directory",
                },
                {
                    "name": BF.README.value,
                    "type": "file",
                    "content": Descriptions.README.value
                }
            ]
        }



from common_components.src.test_project_files import \
    TestProjectFiles as TP
from common_components.src.base_files_dir import \
    BasePythonFileDir as Base
from common_components.src.descriptions import \
    Descriptions as DC
from common_components.src.pytest import \
    PytestFiles


class TestProjectTree:

    @property
    def dir_name(self):
        return self.root_dir_name

    def __init__(self, root_dir_name=None):
        self.root_dir_name = root_dir_name

    @property
    def tree(self):
        return self.tree_dict()

    def tree_dict(self):
        return {
            "name": self.root_dir_name,
            "type": "directory",
            "children": [
                {
                    "name": TP.TOOLS_FOLDER.value,
                    "type": "directory",
                    "children": [
                        {
                            "name": Base.INIT.value,
                            "type": "file",
                            "content": DC.TOOLS_DESCRIPTION.value
                        }
                    ]
                },
                {
                    "name": TP.TESTS_DIR.value,
                    "type": "directory",
                    "children": [
                        {
                            "name": TP.BASE_TEST.value,
                            "type": "file",
                            "content": DC.BASE_TEST.value
                        },
                        {
                            "name": Base.INIT.value,
                            "type": "file"
                        }
                    ]
                },
                {
                    "name": TP.PAGE_OBJECT_DIR.value,
                    "type": "directory",
                    "children": [
                        {
                            "name": TP.POPUPS_DIR.value,
                            "type": "directory",
                            "children": [
                                {
                                    "name": Base.INIT.value,
                                    "type": "file"
                                },
                                {
                                    "name": TP.BASE_POPUP.value,
                                    "type": "file",
                                    "content": DC.BASE_POPUP_EXAMPLE.value
                                }
                            ]
                        },
                        {
                            "name": TP.PAGES_DIR.value,
                            "type": "directory",
                            "children": [
                                {
                                    "name": TP.BASE_PAGE.value,
                                    "type": "file",
                                    "content": DC.BASE_PAGE_EXAMPLE.value
                                },
                                {
                                    "name": Base.INIT.value,
                                    "type": "file"
                                }
                            ]
                        }
                    ]
                },
                {
                    "name": TP.CONFIGS_DIR.value,
                    "type": "directory",
                    "children": [
                        {
                            "name": TP.CONFIG_FILE.value,
                            "type": "file",
                            "content": DC.CONFIG_EXAMPLE.value
                        },
                        {
                            "name": Base.INIT.value,
                            "type": "file"
                        }
                    ]
                },
                {
                    "name": TP.SERVICES_DIR.value,
                    "type": "directory",
                    "children": [
                        {
                            "name": Base.INIT.value,
                            "type": "file"
                        },
                        {
                            "name": TP.BASE_SERVICE.value,
                            "type": "file",
                            "content": DC.BASE_SERVICE_DESCRIPTION.value
                        }
                    ]
                },
                {
                    "name": Base.REQUIREMENTS_FILE.value,
                    "type": "file",
                    "content": DC.REQUIREMENTS_FILE.value
                },
                {
                    "name": PytestFiles.PYTEST_INI_FILE.value,
                    "type": "file",
                    "content": DC.PYTEST_INI_FILE.value
                },
                {
                    "name": PytestFiles.CONFTEST.value,
                    "type": "file",
                    "content": DC.CONFTEST_FILE.value
                },
                {
                    "name": Base.README.value,
                    "type": "file",
                    "content": DC.README.value
                }
            ]
        }



import click
import os
import json
import questionary

from common_components.src.projects_collection import ProjectCollection
from common_components.trees.project_trees import print_dir_tree


class QuestionaryOption(click.Option):

    def __init__(self, param_decls=None, **attrs):
        click.Option.__init__(self, param_decls, **attrs)
        if not isinstance(self.type, click.Choice):
            raise Exception('ChoiceOption type arg must be click.Choice')

    def prompt_for_value(self, ctx):
        val = questionary.select(self.prompt,
                                 choices=self.type.choices).unsafe_ask()
        return val


@click.command()
@click.option("--directory",
              prompt="Name of project root folder. \n"
                     "Skip this step by pressing Enter"
                     " btn if you want to add structure to current directory",
              default="",
              help="Name of directory where project will be created")
@click.option("--project-type", prompt="Select type of project",
              type=click.Choice(["general-tests-project",
                                 "pytest-tests-project"]),
              cls=QuestionaryOption)
@click.option('--docker', prompt="Will you use Docker in your project?",
              type=click.Choice(["yes",
                                 "no"]),
              cls=QuestionaryOption)
@click.option('--jenkins', prompt="Will you use Jenkins in your project?",
              type=click.Choice(["yes",
                                 "no"]),
              cls=QuestionaryOption)
def create_structure(directory, project_type, docker, jenkins):
    """
    Method to create structure of test project
    :param jenkins: to add files for Jenkins
    :param docker: to add files for Docker
    :param directory: directory where folders and files will be added
    :param project_type: specification for the project
    """
    ProjectCollection(directory=directory if directory else None,
                      project_type=project_type).build_project()
    if docker == "yes":
        ProjectCollection(directory=directory if directory else None,
                          project_type="docker").build_project()
    if jenkins == "yes":
        ProjectCollection(directory=directory if directory else None,
                          project_type="jenkins").build_project()


@click.command()
def print_current_dir_tree():
    print("Print structure")
    print(json.dumps(print_dir_tree(os.path.curdir), indent=4))



from common_components.trees.project_trees import ProjectTree
from common_components.trees.env_and_deploy_files import\
    DockerTree, JenkinsTree
from common_components.trees.test_project_sample import\
    TestProjectSampleTree
from common_components.trees.test_project_tree import \
    TestProjectTree


class ProjectCollection:

    def __init__(self, directory, project_type):
        self.project_type = project_type
        self.directory = directory

    def get_project_structure(self):
        """
        This method to store objects of projects trees
        :return: object of tree for particular project
        """
        projects = {"general-tests-project":
                    TestProjectSampleTree(self.directory),
                    "pytest-tests-project":
                        TestProjectTree(self.directory),
                    "docker": DockerTree(self.directory),
                    "jenkins": JenkinsTree(self.directory)}
        return projects.get(self.project_type)

    def build_project(self):
        project = self.get_project_structure()
        return ProjectTree(project).create_project_tree(project.tree,
                                                        project.root_dir_name)



from enum import Enum


class BasePythonFileDir(Enum):
    INIT = "__init__.py"
    README = "README.md"
    REQUIREMENTS_FILE = "requirements.txt"
    DOCKER_FILE = "Dockerfile"
    DOCKER_COMPOSE_FILE = "docker-compose.yml"
    JENKINSFILE = "Jenkinsfile"



from setuptools import find_packages, setup

with open("README.md", "r") as rf:
    long_description = rf.read()

requirements = [
    "click==7.1.2",
    "questionary==1.8.0"
]

setup(
    name="py-init-structure",
    packages=find_packages(),
    version="0.1.3",
    long_description=long_description,
    long_description_content_type="text/markdown",
    description="Creates structure of autotest project",
    author="Sergii Golovach",
    license="MIT",
    classifiers=["Programming Language :: Python :: 3"],
    install_requires=requirements,
    entry_points='''
            [console_scripts]
            init_structure=common_components.src.controller:create_structure
            dir_tree_to_json=common_components.src.controller:print_current_dir_tree
        ''',
    python_requires='>=3.7.5',
)



# flake8: noqa

from enum import Enum


class Descriptions(Enum):
    CONFIG_EXAMPLE = """# store here all  environment variables. Refer to example below: here is var USERNAME that
# could be loaded from
# .env file, or John will be used as a default one
# import os
# from os.path import join, dirname
# from dotenv import load_dotenv

# dotenv_path = join(dirname(__file__), ".env")
# load_dotenv(dotenv_path)

# USERNAME = os.environ.get("USERNAME", "John")

    """

    BASE_POPUP_EXAMPLE = """# Store here common locators and methods for popups of target APP to build Page Object
# structure.
# From this class, all other popups classes will be inherited. NOTE: in example below imports
# are ignored for simplicity

# class BasePopup:
#  POPUP_MODAL = (By.CSS_SELECTOR, "some.css.locator")

#  def __init__(self, driver):
#      self.driver = driver

#  def is_popup_displayed(self):
#     pass

            """

    BASE_PAGE_EXAMPLE = """# Store here common locators and methods for pages of target APP to build
# Page Object structure.
# From this class, all other page classes will be inherited. NOTE: in example below imports
# are ignored for simplicity

# class BasePage:
#   MENU_LIST = (By.CSS_SELECTOR, "some.css.locator.on.page")

#   def __init__(self, driver):
#       self.driver = driver

#  def is_main_menu_displayed(self):
#      pass

    """

    BASE_TEST = """# Create here base class for tests, from which every test will be inherited.
# Example:
# @pytest.mark.usefixtures('preconditions', 'postconditions', 'report_generation')
# class BaseTest:
#       pass

    """

    TOOLS_DESCRIPTION = """# Put here any scripts, emulators, stubs other useful staff, that required
# for test environment or test
# execution, but not directly related to target app

"""

    BASE_SERVICE_DESCRIPTION = """# Create here base class for the application API. Store payloads, routes and methods
#             Example of base service class:
# class BaseService:
#
#     def __init__(self):
#         self.route = ""
#         self.uri = ""
#         self.header = ""
#
#     def post(self, payload: dict, headers, *routes):
#         pass

    """

    REQUIREMENTS_FILE = """ # Put in this file all libs that required for environment
# Example:
# setuptools==50.3.2
# wheel==0.35.1
# twine==3.2.0

"""

    CONFTEST_FILE = """ # External plugin loading: conftest.py is used to import external plugins or modules.
# By defining the following global variable, pytest will load the module and make it available for its test.
# Plugins are generally files defined in your project or other modules which might be needed in your tests.

    """

    PYTEST_INI_FILE = """ # This file is for basic test configuration.
# Refer to https://docs.pytest.org/en/2.7.3/customize.html

    """

    DOCKER_FILE = """ # In case if the test project is supposed to be executed on other machines or via CI tools,
# (Jenkins for example), it would be handy to put code into a docker container. Dockerfile is required for image
# creation. Refer to docker documentation https://docs.docker.com/engine/reference/builder/

    """

    DOCKER_COMPOSE_FILE = """ # In case if more than one container is creating for the test project, compose could be
# used. Compose is a tool for defining and running multi-container Docker applications. With Compose, you use a YAML
# file to configure your application’s services. Then, with a single command, you create and start all the services
# from your configuration. To learn more about all the features of Compose, see the list of features.
# Refer to https://docs.docker.com/compose/

    """

    JENKINSFILE = """ # In case if project us supposed to be executed on CI (Jenkins), Jenkins Pipeline is handy to
# create and support jobs "as code". Purpose of Jenkinsfile - store Jenkins Pipeline script. Get more from the Jenkins
# documentation -  https://www.jenkins.io/doc/book/pipeline/getting-started/

    """

    README = """<Name of the project>.
<Project description>

## Table of Contents
* [Get Started](#get-started)
* [Test Run](#test-run)
* [Documentation](#documentation)

## Get Started <a name="get-started">

## Test Run <a name="test-run">

## Documentation <a name="documentation">
"""



import os


class ProjectTree:
    def __init__(self, project_tree, root_dir=None):
        self.project_tree = project_tree
        self.root_dir = root_dir

    def create_project_tree(self, tree: dict, root_name=None):
        if not root_name:
            root_name = os.getcwd()

        os.makedirs(f"{root_name}", exist_ok=True)
        if tree.get("children"):
            for i in tree.get("children"):
                if i.get("type") == "directory":
                    self.create_project_tree(i, root_name + "/" +
                                             i.get("name"))
                else:
                    with open(os.path.join(root_name,
                                           i.get("name")), 'a') as temp_file:
                        if i.get("content"):
                            temp_file.write(i.get("content"))
                        else:
                            temp_file.write("")


def print_dir_tree(dirname, path=os.path.pathsep):
    data = []
    for name in os.listdir(dirname):
        dct = {'name': name}

        full_path = os.path.join(dirname, name)
        if os.path.isfile(full_path):
            dct['type'] = 'file'
        elif os.path.isdir(full_path):
            dct['type'] = 'folder'
            dct['children'] = print_dir_tree(
                full_path, path=path + name + os.path.pathsep)
        data.append(dct)
    return data



from common_components.src.base_files_dir import \
    BasePythonFileDir as Base

from common_components.src.descriptions import \
    Descriptions as DC


class DockerTree:

    @property
    def dir_name(self):
        return self.root_dir_name

    def __init__(self, root_dir_name=None):
        self.root_dir_name = root_dir_name

    @property
    def tree(self):
        return self.tree_dict()

    def tree_dict(self):
        return {
            "name": self.root_dir_name,
            "type": "directory",
            "children": [
                {
                    "name": Base.DOCKER_COMPOSE_FILE.value,
                    "type": "file",
                    "content": DC.DOCKER_COMPOSE_FILE.value
                },
                {
                    "name": Base.DOCKER_FILE.value,
                    "type": "file",
                    "content": DC.DOCKER_FILE.value
                },
                {
                    "name": Base.JENKINSFILE.value,
                    "type": "file",
                    "content": DC.JENKINSFILE.value
                }
            ]
        }


class JenkinsTree:

    @property
    def dir_name(self):
        return self.root_dir_name

    def __init__(self, root_dir_name=None):
        self.root_dir_name = root_dir_name

    @property
    def tree(self):
        return self.tree_dict()

    def tree_dict(self):
        return {
            "name": self.root_dir_name,
            "type": "directory",
            "children": [
                {
                    "name": Base.JENKINSFILE.value,
                    "type": "file",
                    "content": DC.JENKINSFILE.value
                }
            ]
        }

